/*  Public Domain / CC-0
 *  Author: tastytea <tastytea@tastytea.de>
 */

#include <iostream>
#include <basedir.h>
#include "xdgcfg.hpp"

using std::cerr;
using std::endl;

xdgcfg::xdgcfg(const string &filename, const string &subdir)
: _cfg()
, _verbose(false)
{
    xdgHandle xdg;
    xdgInitHandle(&xdg);
    _filepath = xdgConfigHome(&xdg);
    xdgWipeHandle(&xdg);

    if (!subdir.empty())
    {
        _filepath /= subdir;
    }
    if (!fs::exists(_filepath))
    {
        fs::create_directories(_filepath);
    }
    _filepath /= filename;
}

const uint_fast8_t xdgcfg::read()
{
    try
    {
        _cfg.readFile(_filepath.c_str());
    }
    catch (const libconfig::FileIOException &e)
    {
        if (_verbose)
        {
            cerr << "I/O error while reading " << _filepath
                 << " - " << e.what() << endl;
        }
        return 1;
    }
    catch (const libconfig::ParseException &e)
    {
        if (_verbose)
        {
            cerr << "Parse error at " << e.getFile() << ":" << e.getLine()
                 << " - " << e.getError() << endl;
        }
        return 2;
    }

    return 0;
}

const bool xdgcfg::write()
{
    try
    {
        _cfg.writeFile(_filepath.c_str());
    }
    catch (const libconfig::FileIOException &e)
    {
        if (_verbose)
        {
            cerr << "I/O error while writing " << _filepath
                 << " - " << e.what() << endl;
        }
        return false;
    }

    return true;
}

libconfig::Config &xdgcfg::get_cfg()
{
    return _cfg;
}

const fs::path xdgcfg::get_filepath() const
{
    return _filepath;
}

const void xdgcfg::set_verbose(bool verbose)
{
    _verbose = verbose;
}

const bool xdgcfg::get_verbose() const
{
    return _verbose;
}
